@@ -7,10 +7,10 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '4.1.1'
+__version__ = '5.0.3'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
-__copyright__ = "Copyright 2018-2023, SISSA"
+__copyright__ = "Copyright 2018-2025, SISSA"
 __license__ = "MIT"
 __status__ = "Production/Stable"
 
@@ -23,12 +23,12 @@ from . import protocols  # Protocols for type annotations
 from .exceptions import ElementPathError, MissingContextError, ElementPathKeyError, \
     ElementPathZeroDivisionError, ElementPathNameError, ElementPathOverflowError, \
     ElementPathRuntimeError, ElementPathSyntaxError, ElementPathTypeError, \
-    ElementPathValueError, ElementPathLocaleError
+    ElementPathValueError, ElementPathLocaleError, UnsupportedFeatureError
 
 from .xpath_context import XPathContext, XPathSchemaContext
-from .xpath_nodes import XPathNode, DocumentNode, ElementNode, AttributeNode, \
-    NamespaceNode,  CommentNode, ProcessingInstructionNode, TextNode, \
-    LazyElementNode, SchemaElementNode
+from .xpath_nodes import XPathNode, AttributeNode, NamespaceNode, CommentNode, \
+    ProcessingInstructionNode, TextNode, ElementNode, LazyElementNode, \
+    SchemaElementNode, DocumentNode
 from .tree_builders import get_node_tree, build_node_tree, build_lxml_node_tree, \
     build_schema_node_tree
 from .xpath_tokens import XPathToken, XPathFunction
@@ -36,18 +36,17 @@ from .xpath1 import XPath1Parser
 from .xpath2 import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
 from .schema_proxy import AbstractSchemaProxy
-from .regex import RegexError, translate_pattern
-
-TypedElement = ElementNode  # for backward compatibility with xmlschema<=1.10.0
+from .regex import RegexError, translate_pattern, install_unicode_data, unicode_version
 
 __all__ = ['datatypes', 'protocols', 'etree', 'ElementPathError', 'MissingContextError',
-           'ElementPathKeyError', 'ElementPathZeroDivisionError', 'ElementPathNameError',
+           'UnsupportedFeatureError', 'ElementPathKeyError',  'ElementPathLocaleError',
+           'ElementPathZeroDivisionError', 'ElementPathNameError',
            'ElementPathOverflowError', 'ElementPathRuntimeError', 'ElementPathSyntaxError',
-           'ElementPathTypeError', 'ElementPathValueError', 'ElementPathLocaleError',
-           'XPathContext', 'XPathSchemaContext', 'XPathNode', 'DocumentNode',
-           'ElementNode', 'AttributeNode', 'NamespaceNode', 'CommentNode',
-           'ProcessingInstructionNode', 'TextNode', 'LazyElementNode',
-           'SchemaElementNode', 'TypedElement', 'get_node_tree', 'build_node_tree',
-           'build_lxml_node_tree', 'build_schema_node_tree', 'XPathToken',
+           'ElementPathTypeError', 'ElementPathValueError', 'XPathContext',
+           'XPathSchemaContext', 'XPathNode', 'AttributeNode', 'NamespaceNode',
+           'CommentNode', 'ProcessingInstructionNode', 'TextNode', 'ElementNode',
+           'LazyElementNode', 'SchemaElementNode', 'DocumentNode', 'get_node_tree',
+           'build_node_tree', 'build_lxml_node_tree', 'build_schema_node_tree', 'XPathToken',
            'XPathFunction', 'XPath1Parser', 'XPath2Parser', 'select', 'iter_select',
-           'Selector', 'AbstractSchemaProxy', 'RegexError', 'translate_pattern']
+           'Selector', 'AbstractSchemaProxy', 'RegexError', 'translate_pattern',
+           'install_unicode_data', 'unicode_version']
