@@ -26,7 +26,12 @@ from elementpath import XPathContext, DocumentNode, ElementNode, datatypes, \
 
 class DummyXsdType:
     name = local_name = None
+    xsd_version = '1.0'
 
+    @property
+    def root_type(self): return self
+    @property
+    def simple_type(self): return self
     def is_matching(self, name, default_namespace): pass
     def is_empty(self): pass
     def is_simple(self): pass
@@ -172,14 +177,14 @@ class XPathContextTest(unittest.TestCase):
         node = TextNode('hello world!')
 
         context = XPathContext(self.root, default_collection=1)
-        self.assertListEqual(context.default_collection, [1])
+        self.assertListEqual(context.default_collection, [])
         context = XPathContext(self.root, default_collection=[node])
         self.assertListEqual(context.default_collection, [node])
 
     def test_is_principal_node_kind(self):
         root = ElementTree.XML('<A a1="10" a2="20"/>')
         context = XPathContext(root)
-        self.assertTrue(hasattr(context.item.elem, 'tag'))
+        self.assertTrue(hasattr(context.item.obj, 'tag'))
         self.assertTrue(context.is_principal_node_kind())
         context.item = context.root.attributes[0]
         self.assertFalse(context.is_principal_node_kind())
@@ -239,23 +244,20 @@ class XPathContextTest(unittest.TestCase):
             self.assertListEqual(list(context.iter_attributes()), context.root.attributes)
             self.assertNotEqual(attributes, context.root.attributes)
 
-        context.item = None
-        self.assertListEqual(list(context.iter_attributes()), [])
-
     def test_iter_children_or_self(self):
         doc = ElementTree.ElementTree(self.root)
         context = XPathContext(doc)
         self.assertIsInstance(context.root, DocumentNode)
         self.assertIsInstance(context.root[0], ElementNode)
 
-        self.assertListEqual(list(e.elem for e in context.iter_children_or_self()), [self.root])
+        self.assertListEqual(list(e.obj for e in context.iter_children_or_self()), [self.root])
 
         context.item = context.root[0]  # root element
         self.assertListEqual(list(context.iter_children_or_self()),
                              [context.root[0].children[0]])
 
         context.item = context.root  # document node
-        self.assertListEqual(list(e.elem for e in context.iter_children_or_self()), [self.root])
+        self.assertListEqual(list(e.obj for e in context.iter_children_or_self()), [self.root])
 
     def test_iter_parent(self):
         root = ElementTree.XML('<A a1="10" a2="20"/>')
@@ -275,12 +277,12 @@ class XPathContextTest(unittest.TestCase):
         self.assertListEqual(list(context.iter_parent()), [])
 
         context = XPathContext(root, item=root[2][0])
-        self.assertListEqual(list(e.elem for e in context.iter_parent()), [root[2]])
+        self.assertListEqual(list(e.obj for e in context.iter_parent()), [root[2]])
 
         with patch.object(DummyXsdType(), 'is_empty', return_value=True) as xsd_type:
             context = XPathContext(root, item=root[2][0])
             context.root[2][0].xsd_type = xsd_type
-            self.assertListEqual(list(e.elem for e in context.iter_parent()), [root[2]])
+            self.assertListEqual(list(e.obj for e in context.iter_parent()), [root[2]])
 
     def test_iter_siblings(self):
         root = ElementTree.XML('<A><B1><C1/></B1><B2/><B3><C1/></B3><B4/><B5/></A>')
@@ -289,23 +291,23 @@ class XPathContextTest(unittest.TestCase):
         self.assertListEqual(list(context.iter_siblings()), [])
 
         context = XPathContext(root, item=root[2])
-        self.assertListEqual(list(e.elem for e in context.iter_siblings()), list(root[3:]))
+        self.assertListEqual(list(e.obj for e in context.iter_siblings()), list(root[3:]))
 
         with patch.object(DummyXsdType(), 'is_element_only', return_value=True) as xsd_type:
             context = XPathContext(root, item=root[2])
             context.root[2].xsd_type = xsd_type
-            self.assertListEqual(list(e.elem for e in context.iter_siblings()), list(root[3:]))
+            self.assertListEqual(list(e.obj for e in context.iter_siblings()), list(root[3:]))
 
         context = XPathContext(root, item=root[2])
         self.assertListEqual(
-            list(e.elem for e in context.iter_siblings('preceding-sibling')), list(root[:2])
+            list(e.obj for e in context.iter_siblings('preceding-sibling')), list(root[:2])
         )
 
         with patch.object(DummyXsdType(), 'is_element_only', return_value=True) as xsd_type:
             context = XPathContext(root, item=root[2])
             context.root[2].xsd_type = xsd_type
             self.assertListEqual(
-                list(e.elem for e in context.iter_siblings('preceding-sibling')), list(root[:2])
+                list(e.obj for e in context.iter_siblings('preceding-sibling')), list(root[:2])
             )
 
     @unittest.skipIf(lxml_etree is None, 'lxml library is not installed')
@@ -321,7 +323,7 @@ class XPathContextTest(unittest.TestCase):
         self.assertIsNone(root.getparent())
 
         context = XPathContext(root)
-        self.assertIs(context.root.value, root)
+        self.assertIs(context.root.obj, root)
         self.assertIsInstance(context.document, DocumentNode)
 
         parser = lxml_html.HTMLParser()
@@ -329,7 +331,7 @@ class XPathContextTest(unittest.TestCase):
         self.assertIsNotNone(root.getparent())
 
         context = XPathContext(root)
-        self.assertIs(context.root.value, root)
+        self.assertIs(context.root.obj, root)
         self.assertIsInstance(context.document, DocumentNode)
 
     def test_iter_descendants(self):
@@ -337,7 +339,7 @@ class XPathContextTest(unittest.TestCase):
         context = XPathContext(root)
         attr = context.root.attributes[0]
 
-        self.assertListEqual(list(e.elem for e in context.iter_descendants()),
+        self.assertListEqual(list(e.obj for e in context.iter_descendants()),
                              [root, root[0], root[1]])
 
         context.item = attr
@@ -350,7 +352,7 @@ class XPathContextTest(unittest.TestCase):
             context = XPathContext(root, item=root)
             context.root.xsd_type = xsd_type
             self.assertListEqual(
-                list(e.elem for e in context.iter_descendants()), [root, root[0], root[1]]
+                list(e.obj for e in context.iter_descendants()), [root, root[0], root[1]]
             )
 
     def test_iter_ancestors(self):
@@ -363,7 +365,7 @@ class XPathContextTest(unittest.TestCase):
         context.item = attr
         self.assertListEqual(list(context.iter_ancestors()), [context.root])
 
-        result = list(e.elem for e in XPathContext(root, item=root[1]).iter_ancestors())
+        result = list(e.obj for e in XPathContext(root, item=root[1]).iter_ancestors())
         self.assertListEqual(result, [root])
         with patch.object(DummyXsdType(), 'has_mixed_content', return_value=True) as xsd_type:
             context = XPathContext(root, item=root[1])
@@ -388,7 +390,7 @@ class XPathContextTest(unittest.TestCase):
 
         root = ElementTree.XML('<A><B1><C1/></B1><B2/><B3><C1/><C2/></B3></A>')
         context = XPathContext(root, item=root[2][1])
-        self.assertListEqual(list(e.elem for e in context.iter_preceding()),
+        self.assertListEqual(list(e.obj for e in context.iter_preceding()),
                              [root[0], root[0][0], root[1], root[2][0]])
 
     def test_iter_following(self):
@@ -402,16 +404,16 @@ class XPathContextTest(unittest.TestCase):
         self.assertListEqual(list(context.iter_followings()), [])
 
         context = XPathContext(root, item=root[2])
-        self.assertListEqual(list(e.elem for e in context.iter_followings()), list(root[3:]))
+        self.assertListEqual(list(e.obj for e in context.iter_followings()), list(root[3:]))
 
         context = XPathContext(root, item=root[1])
         result = [root[2], root[2][0], root[3], root[4]]
-        self.assertListEqual(list(e.elem for e in context.iter_followings()), result)
+        self.assertListEqual(list(e.obj for e in context.iter_followings()), result)
 
         with patch.object(DummyXsdType(), 'has_mixed_content', return_value=True) as xsd_type:
             context = XPathContext(root, item=root[1])
             context.root[1].xsd_type = xsd_type
-            self.assertListEqual(list(e.elem for e in context.iter_followings()), result)
+            self.assertListEqual(list(e.obj for e in context.iter_followings()), result)
 
 
 if __name__ == '__main__':
